/*
 * Copyright (C) 1999-2017 Hartmut Seichter
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ifndef PW_SCENE_NODE_HPP
#define PW_SCENE_NODE_HPP

#include <pw/core/globals.hpp>
#include <pw/scene/component.hpp>

#include <string>
#include <vector>

namespace pw {

/**
 * @brief nodes represent the structure for the scene graph
 */
class node {
public:

    typedef shared_ptr<node> ref;
    typedef std::vector<ref> ref_array;
	typedef std::vector<node*> ptr_array;

    friend class component;

    //! standard c'tor
    node(const std::string& name = "");

    //! copy c'tor
    node(const node& node);

	//! d'tor
	virtual ~node();

    //! cloning interface
	virtual ref clone(const unsigned short& copymode) const;

	std::string name() const;	//!< get name
	void set_name(const std::string &name); //!< set name

    //! return list of parents
    inline const ptr_array& parents() const { return _parents; }

    //! add a child node
	ref add_child(ref anode);

    //! get the list of children
    inline const ref_array& children() const { return _children; }

    //! get number of children
    inline size_t child_count() const { return children().size(); }

    //! check if this node is a leaf node
    inline bool is_leaf() const { return children().empty(); }

	//! check if this is the root node
	inline bool is_root() const { return parents().empty(); }

	//! list of components
    const component::array& components() const;

	//! add a node component
	void add_component(component::ref c);

	//! remove a node component
	void remove_component(component::ref c);

protected:

    ref_array _children;
    ptr_array _parents;

    component::array _components;

    std::string _name;

private:

	//! add a node as parent node
	void add_parent(node *anode);

};

}

#endif
